__title__ = 'alpha-vantage-py'
__description__ = 'Alpha Vantage Python package.'
__url__ = 'https://github.com/wstolk/vantage-py'
__version__ = '0.0.3'
__build__ = 0x022501
__author__ = 'Wouter Stolk'
__author_email__ = 'stolk.wouter@gmail.com'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2021 Wouter Stolk'
__cake__ = u'\u2728 \U0001f370 \u2728'


from requests import Session
from urllib.parse import urljoin


class Client(Session):
    def __init__(self, api_key: str):
        """
        Sets up basic Alpha Vantage client for making requests.

        :param api_key: API key from Alpha Vantage
        """
        if len(api_key) < 10:
            raise ValueError("Invalid value for parameter: api_key")

        self.prefix_url = "https://www.alphavantage.co/query"
        self.api_key = api_key
        super(Client, self).__init__()

    def request(self, method, parameters, *args, **kwargs):
        """
        Returns requests response object

        :param method: GET, POST, PUT, PATH or DELETE
        :param parameters: path after the base URL
        :param args:
        :param kwargs:
        :return: requests response object
        """
        url = urljoin(self.prefix_url, f"{parameters}&apikey={self.api_key}")
        print(url)
        return super(Client, self).request(method, url, *args, **kwargs)



import unittest
import os

from alpha_vantage import Client


class ClientTestCase(unittest.TestCase):
    def test_valid_key(self):
        self.assertIsInstance(Client("123456789abcdefg"), Client)

    def test_invalid_key(self):
        with self.assertRaises(ValueError): Client("")

    def test_no_key(self):
        with self.assertRaises(TypeError): Client()



from datetime import datetime
from collections.abc import Sequence

class MetadataModel:
    def __init__(self, symbol: str, last_refreshed: datetime, timezone: str):
        self.symbol = symbol
        self.last_refreshed = last_refreshed
        self.timezone = timezone



class TimeSerieModel:
    def __init__(self, timestamp: datetime, open: float, high: float, low: float, close: float, volume: int):
        self.timestamp = timestamp
        self.open = open
        self.high = high
        self.low = low
        self.close = close
        self.volume = volume


class TimeSeriesModel:
    def __init__(self, series_data: Sequence[TimeSerieModel], metadata: MetadataModel):
        self.metadata = metadata
        self.timeseries = series_data



from datetime import datetime
from alpha_vantage.client import Client
from alpha_vantage.models import MetadataModel, TimeSeriesModel, TimeSerieModel


class TimeSeries:
    """Initialize TimeSeries class for retrieving data from the various timeseries endpoints.

    :param client: API client instance.
    :type client: alpha_vantage.client.Client
    :param symbol: stock ticker object, for example IBM.
    :type symbol: str
    :param outputsize: compact or full, defaults to compact.
    :type outputsize: str
    :param datatype: class or dataframe, defaults to class.
    :type datatype: str
    """

    def __init__(self, client: Client, symbol: str, outputsize: str = "compact", datatype: str = "class"):
        """Constructor method
        """
        self.client = client
        self.url_append = f"symbol={symbol}&outputsize={outputsize}&datatype=json"

    def __process__(self, data: dict, metadata: dict, format: str):
        """Processes input and returns a TimeSeriesModel instance.

        :param data: dict of dict, containing time series data.
        :type data: dict
        :param metadata: dict containing metadata from response, including symbol and timezone.
        :type metadata: dict
        :param format: format for parsing datetime strings to datetime objects.
        :type format: str
        :return: instance of a TimeSeriesModel, including a list of TimeSerieModel data and metadata.
        :rtype: alpha_vantage.models.TimeSeriesModel
        """
        result = []
        metadata = MetadataModel(
            symbol=metadata["2. Symbol"],
            last_refreshed=datetime.strptime(metadata["3. Last Refreshed"], format),
            timezone=metadata["6. Time Zone"] if "6. Time Zone" in metadata else metadata["5. Time Zone"]
        )

        for key in data:
            result.append(TimeSerieModel(
                timestamp=datetime.strptime(key, format),
                open=float(data[key]["1. open"]),
                high=float(data[key]["2. high"]),
                low=float(data[key]["3. low"]),
                close=float(data[key]["4. close"]),
                volume=int(data[key]["5. volume"])
            ))

        return TimeSeriesModel(metadata=metadata, series_data=result)

    def daily(self):
        """Retrieve stock data on a daily level.

        :return: an instance of a TimeSeriesModel, containing the requested stock data and metadata.
        :rtype: alpha_vantage.models.TimeSeriesModel
        """
        url = f"?function=TIME_SERIES_DAILY&{self.url_append}"
        data = self.client.get(url=url).json()
        return self.__process__(data[f"Time Series (Daily)"], metadata=data["Meta Data"], format="%Y-%m-%d")

    def intraday(self, interval: str, adjusted: bool = True):
        """Retrieve stock data on a intradaily level.

        :param interval: interval between the historical data points. Allowed values are 1min, 5min, 15min, 30min,
            60min.
        :type interval: str
        :param adjusted: if True, the output time series is adjusted by historical split and dividend events.
        :type adjusted: bool
        :return: an instance of a TimeSeriesModel, containing the requested stock data and metadata.
        :rtype: alpha_vantage.models.TimeSeriesModel
        """
        if interval not in ['1min', '5min', '15min', '30min', '60min']:
            raise ValueError("Invalid input for parameter interval. Allowed values are 1min, 5min, 15min, 30min, 60min")

        url = f"?function=TIME_SERIES_INTRADAY&adjusted={str(adjusted).lower()}&{self.url_append}"
        data = self.client.get(url=url).json()
        return self.__process__(data=data[f"Time Series ({interval})"], metadata=data["Meta Data"], format="%Y-%m-%d %H:%M:%S")


from .client import Client

from .__version__ import __title__, __description__, __url__, __version__
from .__version__ import __build__, __author__, __author_email__, __license__
from .__version__ import __copyright__, __cake__


import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

packages = ['alpha_vantage']

requires = [
    'requests>=2'
]

test_requirements = []

about = {}
with open(os.path.join(here, 'alpha_vantage', '__version__.py'), 'r') as f:
    exec(f.read(), about)

with open('README.md', 'r') as f:
    readme = f.read()

setup(
    name=about['__title__'],
    packages=find_packages(),
    version=about['__version__'],
    description=about['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    package_data={'': ['LICENSE', 'NOTICE']},
    package_dir={'alpha_vantage': 'alpha_vantage'},
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=requires,
    license=about['__license__'],
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    project_urls={
        'Source': 'https://github.com/wstolk/vantage_py',
    },
)
